@@ -36,18 +36,26 @@
 typedef struct _GeglTileHandlerCache      GeglTileHandlerCache;
 typedef struct _GeglTileHandlerCacheClass GeglTileHandlerCacheClass;
 
+struct _GeglTileHandlerCache
+{
+  GeglTileHandler  parent_instance;
+  GeglTileStorage *tile_storage;
+  GSList          *items;
+  int              count; /* number of items held by cache */
+};
 
 struct _GeglTileHandlerCacheClass
 {
   GeglTileHandlerClass parent_class;
 };
 
-GType                  gegl_tile_handler_cache_get_type (void) G_GNUC_CONST;
-GeglTileHandlerCache * gegl_tile_handler_cache_new      (void);
-void                   gegl_tile_handler_cache_insert   (GeglTileHandlerCache *cache,
-                                                         GeglTile             *tile,
-                                                         gint                  x,
-                                                         gint                  y,
-                                                         gint                  z);
+GType             gegl_tile_handler_cache_get_type (void) G_GNUC_CONST;
+
+GeglTileHandler * gegl_tile_handler_cache_new      (void);
+void              gegl_tile_handler_cache_insert   (GeglTileHandlerCache *cache,
+                                                    GeglTile             *tile,
+                                                    gint                  x,
+                                                    gint                  y,
+                                                    gint                  z);
 
 #endif
