@@ -22,6 +22,11 @@ static const char* op_c_source =
 " * Copyright 2013 Stephan Seifermann <stephan.seifermann@student.kit.edu>     \n"
 " * Copyright 2013 Bastian Pirk <bastian.pirk@student.kit.edu>                 \n"
 " * Copyright 2013 Pascal Giessler <pascal.giessler@student.kit.edu>           \n"
+" * Copyright 2015 Thomas Manni <thomas.manni@free.fr>                         \n"
+" */                                                                           \n"
+"                                                                              \n"
+"/* TODO: Find some better algorithm to calculate the roi for each dest        \n"
+" *       rectangle. Right now it simply asks for the entire image...          \n"
 " */                                                                           \n"
 "                                                                              \n"
 "#include \"config.h\"                                                         \n"
@@ -36,7 +41,7 @@ static const char* op_c_source =
 "  description(_(\"Keep image unchanged, where not affected by the lens.\"))   \n"
 "                                                                              \n"
 "property_color (background_color, _(\"Background color\"), \"none\")          \n"
-"  //ui_meta (\"role\", \"color-secondary\")                                   \n"
+"  ui_meta (\"role\", \"color-secondary\")                                     \n"
 "                                                                              \n"
 "#else                                                                         \n"
 "                                                                              \n"
@@ -45,224 +50,185 @@ static const char* op_c_source =
 "                                                                              \n"
 "#include \"gegl-op.h\"                                                        \n"
 "#include <math.h>                                                             \n"
-"#include <stdio.h>                                                            \n"
+"                                                                              \n"
+"typedef struct                                                                \n"
+"{                                                                             \n"
+"  gfloat  bg_color[4];                                                        \n"
+"  gdouble a, b, c;                                                            \n"
+"  gdouble asqr, bsqr, csqr;                                                   \n"
+"} AlParamsType;                                                               \n"
 "                                                                              \n"
 "/**                                                                           \n"
-" * Computes the projected position (projx, projy) of the                      \n"
-" * original point (x, y) after passing through the lens                       \n"
+" * Computes the original position (ox, oy) of the                             \n"
+" * distorted point (x, y) after passing through the lens                      \n"
 " * which is given by its center and its refraction index.                     \n"
 " * See: Ellipsoid formula: x^2/a^2 + y^2/b^2 + z^2/c^2 = 1.                   \n"
-" *                                                                            \n"
-" * @param a2 semiaxis a                                                       \n"
-" * @param b2 semiaxis b                                                       \n"
-" * @param c2 semiaxis c                                                       \n"
-" * @param x coordinate x                                                      \n"
-" * @param y coordinate y                                                      \n"
-" * @param refraction refraction index                                         \n"
-" * @param projy inout of the projection x                                     \n"
-" * @param projy inout of the projection y                                     \n"
 " */                                                                           \n"
 "static void                                                                   \n"
-"find_projected_pos (gfloat  a2,                                               \n"
-"                    gfloat  b2,                                               \n"
-"                    gfloat  c2,                                               \n"
-"                    gfloat  x,                                                \n"
-"                    gfloat  y,                                                \n"
-"                    gfloat  refraction,                                       \n"
-"                    gfloat *projx,                                            \n"
-"                    gfloat *projy)                                            \n"
+"find_undistorted_pos (gdouble       x,                                        \n"
+"                      gdouble       y,                                        \n"
+"                      gdouble       refraction,                               \n"
+"                      AlParamsType *params,                                   \n"
+"                      gdouble      *ox,                                       \n"
+"                      gdouble      *oy)                                       \n"
 "{                                                                             \n"
-"  gfloat z;                                                                   \n"
-"  gfloat nxangle, nyangle, theta1, theta2;                                    \n"
-"  gfloat ri1 = 1.0;                                                           \n"
-"  gfloat ri2 = refraction;                                                    \n"
+"  gdouble z;                                                                  \n"
+"  gdouble nxangle, nyangle, theta1, theta2;                                   \n"
+"  gdouble ri1 = 1.0;                                                          \n"
+"  gdouble ri2 = refraction;                                                   \n"
 "                                                                              \n"
-"  z = sqrt ((1 - x * x / a2 - y * y / b2) * c2);                              \n"
+"  z = sqrt ((1 - x * x / params->asqr - y * y / params->bsqr) * params->csqr);\n"
 "                                                                              \n"
 "  nxangle = acos (x / sqrt(x * x + z * z));                                   \n"
-"  theta1 = G_PI / 2 - nxangle;                                                \n"
+"  theta1 = G_PI / 2.0 - nxangle;                                              \n"
 "  theta2 = asin (sin (theta1) * ri1 / ri2);                                   \n"
-"  theta2 = G_PI / 2 - nxangle - theta2;                                       \n"
-"  *projx = x - tan (theta2) * z;                                              \n"
+"  theta2 = G_PI / 2.0 - nxangle - theta2;                                     \n"
+"  *ox = x - tan (theta2) * z;                                                 \n"
 "                                                                              \n"
 "  nyangle = acos (y / sqrt (y * y + z * z));                                  \n"
-"  theta1 = G_PI / 2 - nyangle;                                                \n"
+"  theta1 = G_PI / 2.0 - nyangle;                                              \n"
 "  theta2 = asin (sin (theta1) * ri1 / ri2);                                   \n"
-"  theta2 = G_PI / 2 - nyangle - theta2;                                       \n"
-"  *projy = y - tan (theta2) * z;                                              \n"
+"  theta2 = G_PI / 2.0 - nyangle - theta2;                                     \n"
+"  *oy = y - tan (theta2) * z;                                                 \n"
 "}                                                                             \n"
 "                                                                              \n"
-"/**                                                                           \n"
-" * Prepare function of gegl filter.                                           \n"
-" * @param operation given Gegl operation                                      \n"
-" */                                                                           \n"
 "static void                                                                   \n"
 "prepare (GeglOperation *operation)                                            \n"
 "{                                                                             \n"
-"  const Babl *format = gegl_operation_get_source_format (operation, \"input\");\n"
+"  GeglProperties *o = GEGL_PROPERTIES (operation);                            \n"
+"  const Babl *format = babl_format (\"RGBA float\");                          \n"
+"                                                                              \n"
+"  GeglRectangle  *whole_region;                                               \n"
+"  AlParamsType   *params;                                                     \n"
+"                                                                              \n"
+"  if (! o->user_data)                                                         \n"
+"    o->user_data = g_slice_new0 (AlParamsType);                               \n"
+"                                                                              \n"
+"  params = (AlParamsType *) o->user_data;                                     \n"
+"                                                                              \n"
+"  whole_region = gegl_operation_source_get_bounding_box (operation, \"input\");\n"
+"                                                                              \n"
+"  if (whole_region)                                                           \n"
+"    {                                                                         \n"
+"      params->a = 0.5 * whole_region->width;                                  \n"
+"      params->b = 0.5 * whole_region->height;                                 \n"
+"      params->c = MIN (params->a, params->b);                                 \n"
+"      params->asqr = params->a * params->a;                                   \n"
+"      params->bsqr = params->b * params->b;                                   \n"
+"      params->csqr = params->c * params->c;                                   \n"
+"    }                                                                         \n"
+"                                                                              \n"
+"  gegl_color_get_pixel (o->background_color, format, params->bg_color);       \n"
 "                                                                              \n"
 "  gegl_operation_set_format (operation, \"input\", format);                   \n"
 "  gegl_operation_set_format (operation, \"output\", format);                  \n"
 "}                                                                             \n"
 "                                                                              \n"
-"/**                                                                           \n"
-" * Returns the cached region. This is an area filter, which acts on the whole image.\n"
-" * @param operation given Gegl operation                                      \n"
-" * @param roi the rectangle of interest                                       \n"
-" * @return result the new rectangle                                           \n"
-" */                                                                           \n"
-"static GeglRectangle                                                          \n"
-"get_cached_region (GeglOperation       *operation,                            \n"
-"                   const GeglRectangle *roi)                                  \n"
+"static void                                                                   \n"
+"finalize (GObject *object)                                                    \n"
 "{                                                                             \n"
-"  GeglRectangle result = *gegl_operation_source_get_bounding_box (operation, \"input\");\n"
-"  return result;                                                              \n"
+"  GeglOperation *op = (void*) object;                                         \n"
+"  GeglProperties *o = GEGL_PROPERTIES (op);                                   \n"
+"                                                                              \n"
+"  if (o->user_data)                                                           \n"
+"    {                                                                         \n"
+"      g_slice_free (AlParamsType, o->user_data);                              \n"
+"      o->user_data = NULL;                                                    \n"
+"    }                                                                         \n"
+"                                                                              \n"
+"  G_OBJECT_CLASS (gegl_op_parent_class)->finalize (object);                   \n"
+"}                                                                             \n"
+"                                                                              \n"
+"static GeglRectangle                                                          \n"
+"get_required_for_output (GeglOperation       *operation,                      \n"
+"                         const gchar         *input_pad,                      \n"
+"                         const GeglRectangle *roi)                            \n"
+"{                                                                             \n"
+"  GeglRectangle  result = {0,0,0,0};                                          \n"
+"  GeglRectangle *in_rect = gegl_operation_source_get_bounding_box (operation, \"input\");\n"
+"                                                                              \n"
+"  if (!in_rect)                                                               \n"
+"    return result;                                                            \n"
+"  else                                                                        \n"
+"    return *in_rect;                                                          \n"
 "}                                                                             \n"
 "                                                                              \n"
-"/**                                                                           \n"
-" * Process the gegl filter                                                    \n"
-" * @param operation the given Gegl operation                                  \n"
-" * @param input the input buffer.                                             \n"
-" * @param output the output buffer.                                           \n"
-" * @param result the region of interest.                                      \n"
-" * @param level the level of detail                                           \n"
-" * @return True, if the filter was successfull applied.                       \n"
-" */                                                                           \n"
 "static gboolean                                                               \n"
 "process (GeglOperation       *operation,                                      \n"
 "         GeglBuffer          *input,                                          \n"
 "         GeglBuffer          *output,                                         \n"
-"         const GeglRectangle *result,                                         \n"
+"         const GeglRectangle *roi,                                            \n"
 "         gint                 level)                                          \n"
 "{                                                                             \n"
-"  GeglProperties         *o = GEGL_PROPERTIES (operation);                    \n"
-"  const Babl  *input_format = gegl_buffer_get_format (input);                 \n"
-"  const int bytes_per_pixel = babl_format_get_bytes_per_pixel (input_format); \n"
+"  GeglProperties       *o = GEGL_PROPERTIES (operation);                      \n"
+"  AlParamsType    *params = (AlParamsType *) o->user_data;                    \n"
+"  const Babl      *format = babl_format (\"RGBA float\");                     \n"
 "                                                                              \n"
-"  /**                                                                         \n"
-"   * src_buf, dst_buf:                                                        \n"
-"   * Input- and output-buffers, containing the whole selection,               \n"
-"   * the filter should be applied on.                                         \n"
-"   *                                                                          \n"
-"   * src_pixel, dst_pixel:                                                    \n"
-"   * pointers to the current source- and destination-pixel.                   \n"
-"   * Using these pointers, the reading and writing can be delayed till the    \n"
-"   * end of the loop. Hence src_pixel can also point to background_color if   \n"
-"   * necessary.                                                               \n"
-"   */                                                                         \n"
-"  guint8    *src_buf, *dst_buf, *src_pixel, *dst_pixel, background_color[bytes_per_pixel];\n"
+"  GeglSampler        *sampler;                                                \n"
+"  GeglBufferIterator *iter;                                                   \n"
+"  gint                x, y;                                                   \n"
 "                                                                              \n"
-"  /**                                                                         \n"
-"   * (x, y): Position of the pixel we compute at the moment.                  \n"
-"   * (width, height): Dimensions of the lens                                  \n"
-"   * pixel_offset: For calculating the pixels position in src_buf / dst_buf.  \n"
-"   */                                                                         \n"
-"  gint         x, y,                                                          \n"
-"               width, height,                                                 \n"
-"               pixel_offset, projected_pixel_offset;                          \n"
+"  sampler = gegl_buffer_sampler_new_at_level (input, format,                  \n"
+"                                              GEGL_SAMPLER_CUBIC, level);     \n"
 "                                                                              \n"
-"  /**                                                                         \n"
-"   * Further parameters that are needed to calculate the position of a projected\n"
-"   * further pixel at (x, y).                                                 \n"
-"   */                                                                         \n"
-"  gfloat       a, b, c,                                                       \n"
-"               asqr, bsqr, csqr,                                              \n"
-"               dx, dy, dysqr,                                                 \n"
-"               projected_x, projected_y,                                      \n"
-"               refraction_index;                                              \n"
+"  iter = gegl_buffer_iterator_new (output, roi, level, format,                \n"
+"                                   GEGL_ACCESS_WRITE, GEGL_ABYSS_NONE);       \n"
 "                                                                              \n"
-"  gboolean     keep_surroundings;                                             \n"
+"  gegl_buffer_iterator_add (iter, input, roi, level, format,                  \n"
+"                            GEGL_ACCESS_READ, GEGL_ABYSS_NONE);               \n"
 "                                                                              \n"
-"  width = result->width;                                                      \n"
-"  height = result->height;                                                    \n"
-"                                                                              \n"
-"  refraction_index = o->refraction_index;                                     \n"
-"  keep_surroundings = o->keep_surroundings;                                   \n"
-"  gegl_color_get_pixel (o->background_color, input_format, background_color); \n"
-"                                                                              \n"
-"  a = 0.5 * width;                                                            \n"
-"  b = 0.5 * height;                                                           \n"
-"  c = MIN (a, b);                                                             \n"
-"  asqr = a * a;                                                               \n"
-"  bsqr = b * b;                                                               \n"
-"  csqr = c * c;                                                               \n"
-"                                                                              \n"
-"  /**                                                                         \n"
-"   * Todo: We might want to change the buffers sizes, as the memory consumption\n"
-"   * could be rather large.However, due to the lens, it might happen, that one pixel from the\n"
-"   * images center gets stretched to the whole image, so we will still need   \n"
-"   * at least a src_buf of size (width/2) * (height/2) * 4 to be able to      \n"
-"   * process one quarter of the image.                                        \n"
-"   */                                                                         \n"
-"  src_buf = gegl_malloc (width * height * bytes_per_pixel);                   \n"
-"  dst_buf = gegl_malloc (width * height * bytes_per_pixel);                   \n"
-"                                                                              \n"
-"  gegl_buffer_get (input, result, 1.0, input_format, src_buf,                 \n"
-"                   GEGL_AUTO_ROWSTRIDE, GEGL_ABYSS_NONE);                     \n"
-"                                                                              \n"
-"  for (y = 0; y < height; y++)                                                \n"
+"  while (gegl_buffer_iterator_next (iter))                                    \n"
 "    {                                                                         \n"
-"      /* dy is the current pixels distance (in y-direction) of the lenses center */\n"
-"      dy = -((gfloat)y - b + 0.5);                                            \n"
+"      gfloat *out_pixel = iter->data[0];                                      \n"
+"      gfloat *in_pixel  = iter->data[1];                                      \n"
 "                                                                              \n"
-"      /**                                                                     \n"
-"       * To determine, whether the pixel is within the elliptical region affected\n"
-"       * by the lens, we furthermore need the squared distance. So we calculate it\n"
-"       * once for each row.                                                   \n"
-"       */                                                                     \n"
-"      dysqr = dy * dy;                                                        \n"
-"      for (x = 0; x < width; x++)                                             \n"
+"      for (y = iter->roi->y; y < iter->roi->y + iter->roi->height; y++)       \n"
 "        {                                                                     \n"
-"          /* Again we need to calculate the pixels distance from the lens dx. */\n"
-"          dx = (gfloat)x - a + 0.5;                                           \n"
+"          gdouble dy, dysqr;                                                  \n"
 "                                                                              \n"
-"          /* Given x and y we can now determine the pixels offset in our buffer. */\n"
-"          pixel_offset = (x + y * width) * bytes_per_pixel;                   \n"
+"          dy = -((gdouble) y - params->b + 0.5);                              \n"
+"          dysqr = dy * dy;                                                    \n"
 "                                                                              \n"
-"          /* As described above, we only read and write image data once. */   \n"
-"          dst_pixel = &dst_buf[pixel_offset];                                 \n"
-"                                                                              \n"
-"          if (dysqr < (bsqr - (bsqr * dx * dx) / asqr))                       \n"
+"          for (x = iter->roi->x; x < iter->roi->x + iter->roi->width; x++)    \n"
 "            {                                                                 \n"
-"              /**                                                             \n"
-"               * If (x, y) is inside the affected region, we can find its projected\n"
-"               * position, calculate the projected_pixel_offset and set the src_pixel\n"
-"               * to where we want to copy the pixel-data from.                \n"
-"               */                                                             \n"
-"              find_projected_pos (asqr, bsqr, csqr, dx, dy, refraction_index, \n"
-"                                  &projected_x, &projected_y);                \n"
+"              gdouble dx, dxsqr;                                              \n"
 "                                                                              \n"
-"              projected_pixel_offset = ( (gint)(-projected_y + b) * width +   \n"
-"                                         (gint)( projected_x + a) ) * bytes_per_pixel;\n"
+"              dx = (gdouble) x - params->a + 0.5;                             \n"
+"              dxsqr = dx * dx;                                                \n"
 "                                                                              \n"
-"              src_pixel = &src_buf[projected_pixel_offset];                   \n"
-"            }                                                                 \n"
-"          else                                                                \n"
-"            {                                                                 \n"
-"              /**                                                             \n"
-"               * Otherwise (that is for pixels outside the lens), we could either leave\n"
-"               * the image data unchanged, or set it to a specified 'background_color',\n"
-"               * depending on the user input.                                 \n"
-"               */                                                             \n"
-"              if (keep_surroundings)                                          \n"
-"                src_pixel = &src_buf[pixel_offset];                           \n"
+"              if (dysqr < (params->bsqr - (params->bsqr * dxsqr) / params->asqr))\n"
+"                {                                                             \n"
+"                  /**                                                         \n"
+"                   * If (x, y) is inside the affected region, we can find its original\n"
+"                   * position and fetch the pixel with the sampler            \n"
+"                   */                                                         \n"
+"                  gdouble ox, oy;                                             \n"
+"                  find_undistorted_pos (dx, dy, o->refraction_index, params,  \n"
+"                                        &ox, &oy);                            \n"
+"                                                                              \n"
+"                  gegl_sampler_get (sampler, ox + params->a, params->b - oy,  \n"
+"                                    NULL, out_pixel, GEGL_ABYSS_NONE);        \n"
+"                }                                                             \n"
 "              else                                                            \n"
-"                src_pixel = background_color;                                 \n"
+"                {                                                             \n"
+"                  /**                                                         \n"
+"                   * Otherwise (that is for pixels outside the lens), we could either leave\n"
+"                   * the image data unchanged, or set it to a specified 'background_color',\n"
+"                   * depending on the user input.                             \n"
+"                   */                                                         \n"
+"                  if (o->keep_surroundings)                                   \n"
+"                    memcpy (out_pixel, in_pixel, sizeof (gfloat) * 4);        \n"
+"                  else                                                        \n"
+"                    memcpy (out_pixel, params->bg_color, sizeof (gfloat) * 4);\n"
+"                }                                                             \n"
+"                                                                              \n"
+"              out_pixel += 4;                                                 \n"
+"              in_pixel  += 4;                                                 \n"
 "            }                                                                 \n"
-"                                                                              \n"
-"          /**                                                                 \n"
-"           * At the end, we can copy the src_pixel (which was determined above), to\n"
-"           * dst_pixel.                                                       \n"
-"           */                                                                 \n"
-"          memcpy (dst_pixel, src_pixel, bytes_per_pixel);                     \n"
 "        }                                                                     \n"
-"  }                                                                           \n"
+"    }                                                                         \n"
 "                                                                              \n"
-"  gegl_buffer_set (output, result, 0, gegl_buffer_get_format (output), dst_buf,\n"
-"                   GEGL_AUTO_ROWSTRIDE);                                      \n"
-"                                                                              \n"
-"  gegl_free (dst_buf);                                                        \n"
-"  gegl_free (src_buf);                                                        \n"
+"  g_object_unref (sampler);                                                   \n"
 "                                                                              \n"
 "  return TRUE;                                                                \n"
 "}                                                                             \n"
@@ -270,6 +236,7 @@ static const char* op_c_source =
 "static void                                                                   \n"
 "gegl_op_class_init (GeglOpClass *klass)                                       \n"
 "{                                                                             \n"
+"  GObjectClass             *object_class;                                     \n"
 "  GeglOperationClass       *operation_class;                                  \n"
 "  GeglOperationFilterClass *filter_class;                                     \n"
 "  gchar                    *composition =                                     \n"
@@ -289,12 +256,14 @@ static const char* op_c_source =
 "    \"</node>\"                                                               \n"
 "    \"</gegl>\";                                                              \n"
 "                                                                              \n"
+"  object_class    = G_OBJECT_CLASS (klass);                                   \n"
 "  operation_class = GEGL_OPERATION_CLASS (klass);                             \n"
 "  filter_class    = GEGL_OPERATION_FILTER_CLASS (klass);                      \n"
 "                                                                              \n"
+"  object_class->finalize                   = finalize;                        \n"
 "  operation_class->threaded                = FALSE;                           \n"
 "  operation_class->prepare                 = prepare;                         \n"
-"  operation_class->get_cached_region       = get_cached_region;               \n"
+"  operation_class->get_required_for_output = get_required_for_output;         \n"
 "  filter_class->process                    = process;                         \n"
 "                                                                              \n"
 "  gegl_operation_class_set_keys (operation_class,                             \n"
@@ -302,7 +271,8 @@ static const char* op_c_source =
 "    \"title\",       _(\"Apply Lens\"),                                       \n"
 "    \"categories\",  \"map\",                                                 \n"
 "    \"license\",     \"GPL3+\",                                               \n"
-"    \"description\", _(\"Simulates the optical distoration caused by having an elliptical lens over the image\"),\n"
+"    \"description\", _(\"Simulates the optical distortion caused by having \" \n"
+"                     \"an elliptical lens over the image\"),                  \n"
 "    \"reference-composition\", composition,                                   \n"
 "    NULL);                                                                    \n"
 "}                                                                             \n"
