@@ -19,21 +19,31 @@
 #include "config.h"
 #include <glib/gi18n-lib.h>
 
-#ifdef GEGL_CHANT_PROPERTIES
+#ifdef GEGL_PROPERTIES
+
+property_color (value, _("Color"), "black")
+    description (("The color to render (Idefaults to 'black')"))
+    ui_meta     ("role", "color-primary")
+
+property_format (format, _("Babl Format"), 666)
+    description (_("The babl format of the output"))
 
-gegl_chant_color (value, _("Color"), "black",
-                  _("The color to render (defaults to 'black')"))
 #else
 
-#define GEGL_CHANT_TYPE_POINT_RENDER
-#define GEGL_CHANT_C_FILE           "color.c"
+#define GEGL_OP_POINT_RENDER
+#define GEGL_OP_C_SOURCE color.c
 
-#include "gegl-chant.h"
+#include "gegl-op.h"
 
 static void
 gegl_color_op_prepare (GeglOperation *operation)
 {
-  gegl_operation_set_format (operation, "output", babl_format ("RGBA float"));
+  GeglProperties *o = GEGL_PROPERTIES (operation);
+
+  if (o->format)
+    gegl_operation_set_format (operation, "output", o->format);
+  else
+    gegl_operation_set_format (operation, "output", babl_format ("RGBA float"));
 }
 
 static GeglRectangle
@@ -49,27 +59,21 @@ gegl_color_op_process (GeglOperation       *operation,
                        const GeglRectangle *roi,
                        gint                 level)
 {
-  GeglChantO *o = GEGL_CHANT_PROPERTIES (operation);
-  gfloat     *out_pixel = out_buf;
-  gfloat      color[4];
+  GeglProperties *o = GEGL_PROPERTIES (operation);
+  const Babl *out_format = gegl_operation_get_format (operation, "output");
+  gint        pixel_size = babl_format_get_bytes_per_pixel (out_format);
+  void       *out_color  = alloca(pixel_size);
 
-  gegl_color_get_pixel (o->value, babl_format ("RGBA float"), color);
+  gegl_color_get_pixel (o->value, out_format, out_color);
 
-  while (n_pixels--)
-    {
-      out_pixel[0]=color[0];
-      out_pixel[1]=color[1];
-      out_pixel[2]=color[2];
-      out_pixel[3]=color[3];
+  gegl_memset_pattern (out_buf, out_color, pixel_size, n_pixels);
 
-      out_pixel += 4;
-    }
-  return  TRUE;
+  return TRUE;
 }
 
 
 static void
-gegl_chant_class_init (GeglChantClass *klass)
+gegl_op_class_init (GeglOpClass *klass)
 {
   GeglOperationClass            *operation_class;
   GeglOperationPointRenderClass *point_render_class;
@@ -82,11 +86,12 @@ gegl_chant_class_init (GeglChantClass *klass)
   operation_class->prepare          = gegl_color_op_prepare;
 
   gegl_operation_class_set_keys (operation_class,
-    "name"       , "gegl:color",
+    "name",        "gegl:color",
+    "title",      _("Color"),
     "categories" , "render",
     "description",
       _("Generates a buffer entirely filled with the specified color, "
-        "crop it to get smaller dimensions."),
+        "use gegl:crop to get smaller dimensions."),
     NULL);
 }
 
