@@ -22,22 +22,23 @@
 #include <glib/gi18n-lib.h>
 
 
-#ifdef GEGL_CHANT_PROPERTIES
+#ifdef GEGL_PROPERTIES
 
-gegl_chant_string (values, _("Values"), "", _("list of <number>s"))
+property_string (values, _("Values"), "")
+    description(_("list of <number>s"))
 
 #else
 
-#define GEGL_CHANT_TYPE_POINT_FILTER
-#define GEGL_CHANT_C_FILE       "svg-huerotate.c"
+#define GEGL_OP_POINT_FILTER
+#define GEGL_OP_C_SOURCE svg-huerotate.c
 
-#include "gegl-chant.h"
+#include "gegl-op.h"
 #include <math.h>
 #include <stdlib.h>
 
 static void prepare (GeglOperation *operation)
 {
-  Babl *format = babl_format ("RaGaBaA float");
+  const Babl *format = babl_format ("RaGaBaA float");
 
   gegl_operation_set_format (operation, "input", format);
   gegl_operation_set_format (operation, "output", format);
@@ -48,12 +49,13 @@ process (GeglOperation       *op,
          void                *in_buf,
          void                *out_buf,
          glong                n_pixels,
-         const GeglRectangle *roi)
+         const GeglRectangle *roi,
+         gint                 level)
 {
-  GeglChantO *o   = GEGL_CHANT_PROPERTIES (op);
-  gfloat     *in  = in_buf;
-  gfloat     *out = out_buf;
-  gfloat     *m;
+  GeglProperties *o   = GEGL_PROPERTIES (op);
+  gfloat         *in  = in_buf;
+  gfloat         *out = out_buf;
+  gfloat         *m;
 
   gfloat ma[25] = { 1.0, 0.0, 0.0, 0.0, 0.0,
                     0.0, 1.0, 0.0, 0.0, 0.0,
@@ -62,8 +64,8 @@ process (GeglOperation       *op,
                     0.0, 0.0, 0.0, 0.0, 1.0};
   char        *endptr;
   gfloat       value;
-  const gchar  delimiter=',';
-  const gchar *delimiters=" ";
+  const gchar *delimiter = ",";
+  const gchar *delimiters = " ";
   gchar      **values;
   glong        i;
 
@@ -72,8 +74,8 @@ process (GeglOperation       *op,
   if ( o->values != NULL )
     {
       g_strstrip(o->values);
-      g_strdelimit (o->values, delimiters, delimiter);
-      values = g_strsplit (o->values, &delimiter, 1);
+      g_strdelimit (o->values, delimiters, *delimiter);
+      values = g_strsplit (o->values, delimiter, 1);
       if ( values[0] != NULL )
         {
           value = g_ascii_strtod(values[0], &endptr);
@@ -98,7 +100,7 @@ process (GeglOperation       *op,
 
 
 static void
-gegl_chant_class_init (GeglChantClass *klass)
+gegl_op_class_init (GeglOpClass *klass)
 {
   GeglOperationClass            *operation_class;
   GeglOperationPointFilterClass *point_filter_class;
@@ -109,9 +111,11 @@ gegl_chant_class_init (GeglChantClass *klass)
   point_filter_class->process = process;
   operation_class->prepare = prepare;
 
-  operation_class->name        = "gegl:svg-huerotate";
-  operation_class->categories  = "compositors:svgfilter";
-  operation_class->description = _("SVG color matrix operation svg_huerotate");
+  gegl_operation_class_set_keys (operation_class,
+    "name"       , "gegl:svg-huerotate",
+    "categories" , "compositors:svgfilter",
+    "description", _("SVG color matrix operation svg_huerotate"),
+    NULL);
 }
 
 #endif
