@@ -21,28 +21,30 @@
 #include <glib/gi18n-lib.h>
 
 
-#ifdef GEGL_CHANT_PROPERTIES
+#ifdef GEGL_PROPERTIES
 
 
-gegl_chant_color  (color,    _("Color"),      "rgba(0.0,0.0,0.0,0.0)",
-                             _("Color of paint to use for stroking."))
+property_color  (color,    _("Color"),      "rgba(0.0,0.0,0.0,1.0)")
+    description (_("Color of paint to use for stroking."))
 
-gegl_chant_double (width,    _("Width"),  0.0, 200.0, 2.0,
-                             _("The width of the brush used to stroke the path."))
+property_double (width,    _("Width"),  2.0)
+    value_range (0.0, 200.0)
+    description (_("The width of the brush used to stroke the path."))
 
-gegl_chant_double (opacity,  _("Opacity"),  -2.0, 2.0, 1.0,
-                             _("Opacity of stroke, note, does not behave like SVG since at the moment stroking is done using an airbrush tool."))
+property_double (opacity,  _("Opacity"), 1.0)
+    value_range (-2.0, 2.0)
+    description (_("Opacity of stroke, note, does not behave like SVG since at the moment stroking is done using an airbrush tool."))
 
-gegl_chant_string (transform,_("Transform"), "",
-                             _("svg style description of transform."))
+property_string (transform,_("Transform"), "")
+    description (_("svg style description of transform."))
 
-gegl_chant_path   (d,        _("Vector"),
-                             _("A GeglVector representing the path of the stroke"))
+property_path   (d, _("Vector"), NULL)
+    description (_("A GeglVector representing the path of the stroke"))
 
 #else
 
-#define GEGL_CHANT_TYPE_FILTER
-#define GEGL_CHANT_C_FILE "vector-stroke.c"
+#define GEGL_OP_FILTER
+#define GEGL_OP_C_SOURCE vector-stroke.c
 
 #include "gegl-plugin.h"
 
@@ -52,7 +54,7 @@ static void path_changed (GeglPath *path,
                           const GeglRectangle *roi,
                           gpointer userdata);
 
-#include "gegl-chant.h"
+#include "gegl-op.h"
 #include <cairo.h>
 
 static void path_changed (GeglPath *path,
@@ -60,7 +62,7 @@ static void path_changed (GeglPath *path,
                           gpointer userdata)
 {
   GeglRectangle rect = *roi;
-  GeglChantO    *o   = GEGL_CHANT_PROPERTIES (userdata);
+  GeglProperties    *o   = GEGL_PROPERTIES (userdata);
   /* invalidate the incoming rectangle */
 
   rect.x -= o->width/2;
@@ -74,8 +76,9 @@ static void path_changed (GeglPath *path,
 static void
 prepare (GeglOperation *operation)
 {
-  GeglChantO *o = GEGL_CHANT_PROPERTIES (operation);
-  gegl_operation_set_format (operation, "output", babl_format ("RaGaBaA float"));
+  GeglProperties *o = GEGL_PROPERTIES (operation);
+  gegl_operation_set_format (operation, "output", babl_format ("R'aG'aB'aA float"));
+
   if (o->transform && o->transform[0] != '\0')
     {
       GeglMatrix3 matrix;
@@ -87,7 +90,7 @@ prepare (GeglOperation *operation)
 static GeglRectangle
 get_bounding_box (GeglOperation *operation)
 {
-  GeglChantO    *o       = GEGL_CHANT_PROPERTIES (operation);
+  GeglProperties    *o       = GEGL_PROPERTIES (operation);
   GeglRectangle  defined = { 0, 0, 512, 512 };
   GeglRectangle *in_rect;
   gdouble        x0, x1, y0, y1;
@@ -143,15 +146,17 @@ static gboolean
 process (GeglOperation       *operation,
          GeglBuffer          *input,
          GeglBuffer          *output,
-         const GeglRectangle *result)
+         const GeglRectangle *result,
+         gint                 level)
 {
-  GeglChantO *o = GEGL_CHANT_PROPERTIES (operation);
+  GeglProperties *o = GEGL_PROPERTIES (operation);
   gboolean need_stroke = FALSE;
-  gdouble r,g,b,a;
+  gdouble color[4] = {0, 0, 0, 0};
 
   if (input)
     {
-      gegl_buffer_copy (input, result, output, result);
+      gegl_buffer_copy (input, result, GEGL_ABYSS_NONE,
+                        output, result);
     }
   else
     {
@@ -160,21 +165,21 @@ process (GeglOperation       *operation,
 
   if (o->width > 0.1 && o->opacity > 0.0001)
     {
-      gegl_color_get_rgba (o->color, &r,&g,&b,&a);
-      a *= o->opacity;
-      if (a>0.001)
+      gegl_color_get_pixel (o->color, babl_format ("R'G'B'A double"), color);
+      color[3] *= o->opacity;
+      if (color[3] > 0.001)
           need_stroke=TRUE;
     }
 
   if (need_stroke)
     {
-      GStaticMutex mutex = G_STATIC_MUTEX_INIT;
+      static GMutex mutex = { 0, };
       cairo_t *cr;
       cairo_surface_t *surface;
       guchar *data;
 
-      g_static_mutex_lock (&mutex);
-      data = (void*)gegl_buffer_linear_open (output, result, NULL, babl_format ("B'aG'aR'aA u8"));
+      g_mutex_lock (&mutex);
+      data = gegl_buffer_linear_open (output, result, NULL, babl_format ("cairo-ARGB32"));
       surface = cairo_image_surface_create_for_data (data,
                                                      CAIRO_FORMAT_ARGB32,
                                                      result->width,
@@ -190,12 +195,12 @@ process (GeglOperation       *operation,
       cairo_set_line_join   (cr, CAIRO_LINE_JOIN_ROUND);
 
       gegl_path_cairo_play (o->d, cr);
-      cairo_set_source_rgba (cr, r,g,b,a);
+      cairo_set_source_rgba (cr, color[0], color[1], color[2], color[3]);
       cairo_stroke (cr);
       cairo_destroy (cr);
 
       gegl_buffer_linear_close (output, data);
-      g_static_mutex_unlock (&mutex);
+      g_mutex_unlock (&mutex);
     }
   return  TRUE;
 }
@@ -233,7 +238,7 @@ static GeglNode *detect (GeglOperation *operation,
                          gint           x,
                          gint           y)
 {
-  GeglChantO *o = GEGL_CHANT_PROPERTIES (operation);
+  GeglProperties *o = GEGL_PROPERTIES (operation);
   cairo_t *cr;
   cairo_surface_t *surface;
   gchar *data = "     ";
@@ -258,8 +263,25 @@ static GeglNode *detect (GeglOperation *operation,
   return NULL;
 }
 
+  gchar                    *composition = "<?xml version='1.0' encoding='UTF-8'?>"
+    "<gegl>"
+    "<node operation='gegl:crop' width='200' height='200'/>"
+    "<node operation='gegl:over'>"
+    "<node operation='gegl:translate' x='40' y='40'/>"
+    "<node operation='gegl:vector-stroke'>"
+    "  <params>"
+    "    <param name='color'>rgb(0.0, 0.6, 1.0)</param>"
+    "    <param name='d'>"
+    "M0,50 C0,78 24,100 50,100 C77,100 100,78 100,50 C100,45 99,40 98,35 C82,35 66,35 50,35 C42,35 35,42 35,50 C35,58 42,65 50,65 C56,65 61,61 64,56 C67,51 75,55 73,60 C69,  69 60,75 50,75 C36,75 25,64 25,50 C25,36 36,25 50,25 L93,25 C83,9 67,0 49,0 C25,0 0,20 0,50   z"
+    "                    </param>"
+    "  </params>"
+    "</node>"
+    "</node>"
+    "<node operation='gegl:checkerboard' color1='rgb(0.25,0.25,0.25)' color2='rgb(0.75,0.75,0.75)'/>"
+    "</gegl>";
+
 static void
-gegl_chant_class_init (GeglChantClass *klass)
+gegl_op_class_init (GeglOpClass *klass)
 {
   GeglOperationClass       *operation_class;
   GeglOperationFilterClass *filter_class;
@@ -273,9 +295,13 @@ gegl_chant_class_init (GeglChantClass *klass)
   operation_class->detect = detect;
   /*operation_class->no_cache = TRUE;*/
 
-  operation_class->name        = "gegl:vector-stroke";
-  operation_class->categories  = "render";
-  operation_class->description = _("Renders a vector stroke");
+  gegl_operation_class_set_keys (operation_class,
+    "name",                  "gegl:vector-stroke",
+    "title",                 _("Vector Stroke"),
+    "reference-composition", composition,
+    "categories",            "render",
+    "description",           _("Renders a vector stroke"),
+    NULL);
 }
 
 
